#include "UserAnalysis.h"

ClassImp(UserAnalysis)

//______________________________________________________________________________
// ---------------
//  Ctor and Dtor
// ---------------
//
UserAnalysis::UserAnalysis(BabyDisplay *display = 0) :
	fDisplay(display) {}

UserAnalysis::~UserAnalysis() {}

//______________________________________________________________________________
// --------------
//  Baby Chamber
// --------------
//

void UserAnalysis::InitRawFADC()
{
  TDirectory *last = gDirectory;
  gJSF->GetOutput()->cd("/");
  Char_t st[256];
  for (Int_t i = 0; i < kNmods; i++) {
     sprintf(st,"hFADC%2.2d",i);
#if 0
     if (i < 8) hFADC[i] = new TH1F(st,"",2732,0.,8196.);
     else	hFADC[i] = new TH1F(st,"",4098,500.,8696.);
#else
//     if (i < 8) hFADC[i] = new TH1F(st,"",2108,0.,6324.);
//     else	hFADC[i] = new TH1F(st,"",3162,0.,6324.);
     if (i < 8) hFADC[i] = new TH1F(st,"",2200,0.,6600.);
     else	hFADC[i] = new TH1F(st,"",3300,0.,6600.);
#endif
  }
  last->cd();
}

void UserAnalysis::DrawRawFADC()
{
  TDirectory *last = gDirectory;
  gFile->cd("/");

  if (!(fDisplay->IsInLoop())) {

    CDCRawData     *raw = (CDCRawData*)gJSF->FindModule("CDCRawData");
    CDCRawDataBuf  *evt = (CDCRawDataBuf *)raw->EventBuf();
    CDCBeginRunBuf *br  = raw->GetBeginRunBuf();
    CDCHitMaker    *hm = (CDCHitMaker*)gJSF->FindModule("CDCHitMaker");
    CDCHitBuf      *hb = (CDCHitBuf *)hm->EventBuf();
    TObjArray      *la = hb->GetHitLayers();
    CDCTrackMaker  *tm = (CDCTrackMaker*)gJSF->FindModule("CDCTrackMaker");
    CDCTrackBuf    *tb = (CDCTrackBuf *)tm->EventBuf();
    TObjArray      *ta = tb->GetTracks();

    Int_t              ncdc = evt->GetNADCData();
    TClonesArray   *adcdata = evt->GetADCData();

    for (Int_t i = 0; i < kNmods; i++) {
      hFADC[i]->Reset();
    }
    Int_t iset;
    if (gJSF->GetRunNumber() < 828) 	iset = 0;
    else					iset = 1;
    for (Int_t i = 0; i < ncdc; i++) {
      CDCRawADCDatum *adcdatum = (CDCRawADCDatum *)adcdata->UncheckedAt(i);
      Int_t a;
      Int_t id;
      Int_t cr = 0;               // We have only 1 crate now.
      Int_t st = adcdatum->GetStationNo();
      Int_t ch = adcdatum->GetChannelNo();
      if (st > 13) break;              // TEMPORARY REMEDY FOR BROKEN DATA
      if (st <=13 && ch > 1) continue; // TEMPORARY REMEDY FOR BROKEN DATA
      br->EChannelToDChannel(cr,st,ch,id,a);
      Double_t adc  = 255 - adcdatum->GetADC();
      Double_t time = adcdatum->GetTime();
      if (a < 8) time = (time - kT0[iset][a])*3.;
      else       time = (time - kT0[iset][a])*2.;
      if (a >= 5) time = kFullDriftTime - time;
      if ( id == 0 && a >= 0 && a < kNmods ) {
        hFADC[a]->Fill(time,adc);
      }
    }

    TCanvas *da = (TCanvas *)gROOT->FindObject("DisplayArea");  
    da->cd();
    da->Divide(1,kNmods);
    for (Int_t i = 0; i < kNmods; i++) {
      da->cd(i+1);
      gStyle->SetOptStat(0);
      gPad->SetTopMargin(0.01);
      gPad->SetBottomMargin(0.2);
      hFADC[i]->GetXaxis()->SetTickLength(0.1);
      hFADC[i]->GetYaxis()->SetTickLength(0.005);
      hFADC[i]->GetXaxis()->SetNdivisions(30);
      hFADC[i]->GetYaxis()->SetNdivisions(4); 
      hFADC[i]->GetXaxis()->SetLabelSize(0.20); 
      hFADC[i]->GetYaxis()->SetLabelSize(0.20); 
      hFADC[i]->Draw();
    }
#if 1
    CDCHitLayer *layerp;
    TIter nextLayer(la);
    while ((layerp = (CDCHitLayer *)nextLayer())) {
      TIter nextCell(layerp);
      CDCHitCell *cellp;
      while ((cellp = (CDCHitCell *)nextCell())) {
	TIter nextWire(cellp);
	CDCHitWire *wirep;
	while ((wirep = (CDCHitWire *)nextWire())) {
	  Int_t layerNo = wirep->GetLayerNo();
	  Int_t cellNo  = wirep->GetCellNo();
	  Int_t wireNo  = wirep->GetWireNo();
	  Int_t a       = 5*layerNo + wireNo;  // works only for baby
	  if (cellNo == 1) a = 10 + wireNo;    // works only for baby
	  if (a < 0 || a > 9) continue;        // works only for baby
	  Char_t str[256];
	  sprintf(str,"hHit%2.2d",a);
	  DeleteGraph(str);
	  TGraph *gr = new TGraph();
	  gr->SetName(str);
	  da->cd(a+1);
	  CDCHit *hitp;
	  TIter nextHit(wirep);
	  Int_t nhits = 0;
	  while ((hitp = (CDCHit *)nextHit())) {
	    Double_t time = hitp->GetCluster()->GetStartTime();
	    if (a >= 5) time = kFullDriftTime - time;
	    gr->SetPoint(nhits++,time,0.);
	    gr->SetFillColor(19);
	    gr->SetMarkerColor(1);
	    gr->SetMarkerStyle(23);
	    gr->SetMarkerSize(1.5);
	    gr->Draw("P");
	  }
	}
      }
    }
#endif

#if 1
    CDCTrack *tp;
    TIter nexttrack(ta);
    Int_t ntrks = 0;
    Int_t ntrkhits = 0;
    while ((tp = (CDCTrack *)nexttrack())) {
      ntrks++;
      CDCTrackHit *trkhitp;
      CDCHit *hitp;
      TIter nexttrkhit(tp);
      while ((trkhitp = (CDCTrackHit *)nexttrkhit())) {
	hitp = trkhitp->GetCDCHit();
	Int_t layerNo = hitp->GetLayerNo();
	Int_t cellNo  = hitp->GetCellNo();
	Int_t wireNo  = hitp->GetWireNo();
	Int_t a       = 5*layerNo + wireNo;  // works only for baby
	if (cellNo == 1) a = 10 + wireNo;    // works only for baby
	if (a < 0 || a > 9) continue;        // works only for baby
	Double_t time = hitp->GetCluster()->GetStartTime();
	if (a >= 5) time = kFullDriftTime - time;
	da->cd(a+1);
	Char_t str[256];
	sprintf(str,"hTrackHit%2.2d%3.3d",a,ntrkhits++);
	DeleteGraph(str);
	TGraph *gr = new TGraph();
	gr->SetName(str);
	gr->SetPoint(0,time,0.);
	gr->SetFillColor(19);
	gr->SetMarkerColor((1+ntrks)%50);
	gr->SetMarkerStyle(8);
	gr->SetMarkerSize(0.8);
	gr->Draw("P");
      }
    }
#endif
    da->cd();
    da->Update();
  }
  last->cd();
}

//______________________________________________________________________________
// ------------
//  Hodoscopes
// ------------
//

void UserAnalysis::InitCounters()
{
  TDirectory *last = gDirectory;
  gJSF->GetOutput()->cd("/");
  Char_t st[256];
  for (Int_t i = 0; i < kNhodos-1; i++) {
     sprintf(st,"hHADC%2.2d",i);
     hHADC[i] = new TH1F(st,"",128,0.,2048.);
  }
  hHod1     = new TH1F("hHod1","Hodoscope 1",16,0.,16.);
  hHod2     = new TH1F("hHod2","Hodoscope 2",16,0.,16.);
  hHod12    = new TH2F("hHod12","Hod1 v.s. Hod2",16,0.,16.,16,0.,16.); 
  hHodADC12 = new TH2F("hHodADC12","SumHod1 v.s. SumHod2",
  			128,0.,8192.,128,0.,8192.); 
  hHod12lo = new TH2F("hHodADC12lo","SumHod1 v.s. SumHod2 (single)",
  			128,0.,8192.,128,0.,8192.); 
  hHod12hi = new TH2F("hHodADC12hi","SumHod1 v.s. SumHod2 (double)",
  			128,0.,8192.,128,0.,8192.); 
  hTC1TC2   = new TH2F("hTC1TC2","TC1 v.s. TC2",
  			128,0.,2048.,128,0.,2048.); 
  hMonHod   = new TH2F("hMonHod","Mon v.s. Hod",
  			128,0.,2048.,128,0.,16384.); 
  hHod1hs   = new TH2F("hHod1hs","Hod1: Hard v.s. Soft",
  			128,0.,2048.,128,0.,16384.); 
  hNHod12   = new TH2F("hNHod12","Nhod1 v.s. Nhod2",
  			16,0.,16.,16,0.,16.); 
  hMonCsi   = new TH2F("hMonCsi","Mon v.s. Csi",
  			128,0.,2048.,128,0.,2048.); 
  last->cd();
}

void UserAnalysis::DrawCounters()
{
  TDirectory *last = gDirectory;
  gFile->cd("/");
  CDCRawData     *raw = (CDCRawData*)gJSF->FindModule("CDCRawData");
  CDCRawDataBuf  *evt = (CDCRawDataBuf *)raw->EventBuf();
  CDCBeginRunBuf *br  = raw->GetBeginRunBuf();

  Int_t              ncdc = evt->GetNADCData();
  TClonesArray   *adcdata = evt->GetADCData();

  hHod1->Reset();
  hHod2->Reset();

  Int_t    h1[16], h2[16];
  Double_t adc1[16], adc2[16], adc1pk[18], adc2pk[18];
  for (Int_t i = 0; i < 18; i++) adc1pk[i] = adc2pk[i] = 0;
  Int_t    n1 = 0;
  Int_t    n2 = 0;
  Double_t adcsum1 = 0;
  Double_t adcsum2 = 0;
  Double_t adcmon  = 0;
  Double_t adccsi  = 0;
  Double_t adctc1  = 0;
  Double_t adctc2  = 0;
  Int_t ic1st = fDisplay->SkipChamberData();
  for (Int_t i = ic1st; i < ncdc; i++) {
     CDCRawADCDatum *adcdatum = (CDCRawADCDatum *)adcdata->UncheckedAt(i);
     Int_t a;
     Int_t id;
     Int_t cr = 0;               // We have only 1 crate now.
     Int_t st = adcdatum->GetStationNo();
     Int_t ch = adcdatum->GetChannelNo();
     if (st == 15) cerr << "Station No." << st << " required !" << endl;
     if (st == 15) break;             // TEMPORARY REMEDY FOR BROKEN DATA
     if (st <=13 && ch > 1) continue; // TEMPORARY REMEDY FOR BROKEN DATA
     if (st <=13) continue;	      // skip chamber data
     br->EChannelToDChannel(cr,st,ch,id,a);
     if (id == 0) continue;
     
     Double_t adc  = adcdatum->GetADC();
     if ( id != 0 && a >= 0 && a < kNhodos-1 ) {	// hodoscopes
        if (adc > kHodPed[a] + kHodThresh) {
           hHADC[a]->Fill(adc);
           if (a < kNhodos/2) {
              hHod1->Fill(a,adc);
              adc1[n1]    = adc - kHodPed[a];
	      adc1pk[a+1] = adc1[n1];
              adcsum1    += adc1[n1];
              h1[n1++]    = a;
           } else {
              hHod2->Fill(a%(kNhodos/2),adc);
              adc2[n2]              = adc - kHodPed[a];
              adc2pk[a+1-kNhodos/2] = adc2[n2];
              adcsum2              += adc2[n2];
              h2[n2++]              = a%(kNhodos/2);
           }
        }
     } else if ( id != 0 && a >= kNhodos-1 ) {	// counters
        switch (a) {
       	   case 31: {
       	  	adctc1 = adc;
       	  	break;
       	   }
       	   case 33: {
       	  	adctc2 = adc;
       	  	break;
       	   }
       	   case 34: {
       	  	adcmon = adc;
       	  	break;
       	   }
       	   case 35: {
       	  	adccsi = adc;
       	  	break;
       	   }
       	   default:
       	   	break;
        }
     }
  }

  Int_t npk1 = 0;
  Int_t npk2 = 0;
  for (Int_t i = 1; i <= kNhodos/2; i++) {
     if (adc1pk[i] > adc1pk[i-1] && adc1pk[i] >= adc1pk[i+1]) npk1++;
     if (adc2pk[i] > adc2pk[i-1] && adc2pk[i] >= adc2pk[i+1]) npk2++;
  }
#if 0
  cerr << " npk1 = " << npk1 << " npk2 = " << npk2
       << " adcsum1 = " << adcsum1 << endl;
#endif
  for (Int_t i = 0; i < n1; i++) {
     for (Int_t j = 0; j < n2; j++) {
     	hHod12->Fill(h1[i],h2[j],adc1[i]*adc2[j]);
     }
  }
  hHodADC12->Fill(adcsum1,adcsum2);
  hTC1TC2->Fill(adctc1,adctc2);
  hMonHod->Fill(adcmon,adcsum1+adcsum2);
  hHod1hs->Fill(adcmon,adcsum1);
  hNHod12->Fill(n1+0.5,n2+0.5);
  hMonCsi->Fill(adcmon,adccsi);
  if (npk1 < 2 && npk2 < 2) {
     hHod12lo->Fill(adcsum1,adcsum2);
  } else if ( npk1 > 1 && npk2 > 1) {
     hHod12hi->Fill(adcsum1,adcsum2);
  }
  
  if (!(fDisplay->IsInLoop())) {  
     TCanvas *da = (TCanvas *)gROOT->FindObject("DisplayArea");  
     da->cd();
     Int_t page = fDisplay->GetPage();
     switch (page) {
        case 1: {
           da->Divide(1,2);
           gStyle->SetOptStat(0);
           da->cd(1);
           hHod1->Draw();
           da->cd(2);
           hHod2->Draw();
           break;
        }
        case 2: {
           da->Divide(8,4);
           for (Int_t i = 0; i < kNhodos-1; i++) {
              da->cd(i+1);
              gStyle->SetOptStat(0);
              gPad->SetTopMargin(0.01);
              gPad->SetBottomMargin(0.2);
              hHADC[i]->GetXaxis()->SetTickLength(0.04);
              hHADC[i]->GetYaxis()->SetTickLength(0.05);
              hHADC[i]->GetXaxis()->SetNdivisions(5);
              hHADC[i]->GetYaxis()->SetNdivisions(5); 
              hHADC[i]->GetXaxis()->SetLabelSize(0.10); 
              hHADC[i]->GetYaxis()->SetLabelSize(0.10); 
              hHADC[i]->Draw();
              // da->Update();
           }
           break;
        }
        case 3: {
           da->Divide(3,4);
           gStyle->SetOptStat(0);
           da->cd(1);
           hHod12->Draw();
           da->cd(2);
           DeleteHist("hHod12_px");
           hHod12->ProjectionX()->Draw();
           da->cd(3);
           DeleteHist("hHod12_py");
           hHod12->ProjectionY()->Draw();
           da->cd(4);
           hHodADC12->Draw();
           da->cd(5);
           DeleteHist("hHodADC12_px");
           hHodADC12->ProjectionX()->Draw();
           da->cd(6);
           DeleteHist("hHodADC12_py");
           hHodADC12->ProjectionY()->Draw();
           da->cd(7);
#if 1
           DeleteHist("hTC1TC2_px");
           hTC1TC2->ProjectionX()->Draw();
#else
           hTC1TC2->Draw();
#endif
           da->cd(8);
#if 1
           hHod1hs->Draw();
#else
           hMonHod->Draw();
#endif
           da->cd(9);
           hNHod12->Draw();
           da->cd(10);
           hMonCsi->Draw();
#if 1
	   da->cd(11);
	   hHod12lo->Draw();
	   da->cd(12);
	   hHod12hi->Draw();
#endif
           break;
        }
        default:
           break;
     }
     da->cd(); da->Update();
  }
  last->cd();
}


//______________________________________________________________________________
// ----------
//  Steering
// ----------
//

void UserAnalysis::Initialize()
{
  // cerr << "User's Initialize() is called" << endl;
  InitRawFADC();
  InitCounters();
}

void UserAnalysis::DrawHist()
{
  // cerr << "DrawHist() is called" << endl;
  Int_t page = fDisplay->GetPage();
  switch (page) {
     case 3:
     case 2:
     case 1:
     	DrawCounters();
        break;
     case 0:
  	DrawRawFADC();
  	break;
     default:
        break;
  }
}


//______________________________________________________________________________
// ----------
//  Utility
// ----------
//

void UserAnalysis::DeleteHist(Char_t * name)
{
  TDirectory *last = gDirectory;
  TH1D *h = (TH1D *)gROOT->FindObject(name);
  if (h) delete h;
  last->cd();
}

void UserAnalysis::DeleteGraph(Char_t * name)
{
  TDirectory *last = gDirectory;
  TGraph *gr = (TGraph *)gROOT->FindObject(name);
  if (gr) delete gr;
  last->cd();
}
